/*
  Project:
  File:        macromcu.h
  Description: IO macro for AVR mcu and avrgcc compiler. 
               Use these macro to control IO. 
  Author:      Shaoziyang.
               shaoziyang@gmail.com
  Version:     1.0
  Date:        2011.Mar
  
*/

#ifndef _MACRO_MCU_H_
#define _MACRO_MCU_H_ 1

#include <avr/io.h>

#define PIN_OUTPUT     1
#define PIN_INPUT      0

#define PIN_HIGH       1
#define PIN_LOW        0

#define MACRO_CONCAT2_EXPAND(a, b)  a ## b
#define MACRO_CONCAT2(a, b)         MACRO_CONCAT2_EXPAND(a, b)
#define MACRO_CONCAT3_EXPAND(a, b, c) a ## b ## c
#define MACRO_CONCAT3(a, b, c)      MACRO_CONCAT3_EXPAND(a, b, c)

#define MACRO_ARG21(a, b)           a
#define MACRO_ARG22(a, b)           b


#define PINDIR(pin, DIR) \
        {\
          MACRO_CONCAT2(DDR, MACRO_ARG21(pin)) &= ~(1 << MACRO_ARG22(pin));\
          MACRO_CONCAT2(DDR, MACRO_ARG21(pin)) |= (DIR << MACRO_ARG22(pin));\
        }

#define PINOUT(pin, OUT) \
        { \
          MACRO_CONCAT2(PORT, MACRO_ARG21(pin)) &= ~(1 << MACRO_ARG22(pin));\
          MACRO_CONCAT2(PORT, MACRO_ARG21(pin)) |= (OUT << MACRO_ARG22(pin));\
        }

#define PININ(pin)   (MACRO_CONCAT2(PIN, MACRO_ARG21(pin)) & (1 << MACRO_ARG22(pin)))

#define PINSET(pin)  MACRO_CONCAT2(PORT, MACRO_ARG21(pin)) |= (1 << MACRO_ARG22(pin))
#define PINCLR(pin)  MACRO_CONCAT2(PORT, MACRO_ARG21(pin)) &= ~(1 << MACRO_ARG22(pin))
#define PININV(pin)  MACRO_CONCAT2(PORT, MACRO_ARG21(pin)) ^= (1 << MACRO_ARG22(pin))


#endif
